@@ -119,8 +119,7 @@ adblock_fixup_regexp (const gchar* prefix,
 static void
 adblock_destroy_db ()
 {
-    if (blockcss)
-        g_string_free (blockcss, TRUE);
+    g_string_free (blockcss, TRUE);
     blockcss = NULL;
 
     g_hash_table_destroy (pattern);
@@ -157,8 +156,6 @@ adblock_init_db ()
                    NULL,
                    (GDestroyNotify)g_free);
 
-    if (blockcss && blockcss->len > 0)
-        g_string_free (blockcss, TRUE);
     blockcss = g_string_new ("z-non-exist");
 }
 
@@ -455,14 +452,6 @@ adblock_activate_link_cb (GtkWidget*   label,
 }
 #endif
 
-static void
-adblock_preferences_response_cb (GtkWidget* dialog,
-                                 gint       response,
-                                 gpointer   data)
-{
-    gtk_widget_destroy (dialog);
-}
-
 static GtkWidget*
 adblock_get_preferences_dialog (MidoriExtension* extension)
 {
@@ -496,9 +485,7 @@ adblock_get_preferences_dialog (MidoriExtension* extension)
     dialog = gtk_dialog_new_with_buttons (dialog_title, GTK_WINDOW (browser),
         GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
         #if !HAVE_OSX
-        #if !HAVE_HILDON
         GTK_STOCK_HELP, GTK_RESPONSE_HELP,
-        #endif
         GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
         #endif
         NULL);
@@ -513,8 +500,8 @@ adblock_get_preferences_dialog (MidoriExtension* extension)
                                        GTK_RESPONSE_HELP, FALSE);
     sokoke_widget_get_text_size (dialog, "M", &width, &height);
     gtk_window_set_default_size (GTK_WINDOW (dialog), width * 52, -1);
-    g_signal_connect (dialog, "response",
-                      G_CALLBACK (adblock_preferences_response_cb), NULL);
+    g_signal_connect_swapped (dialog, "response",
+                      G_CALLBACK (gtk_widget_destroy), dialog);
     /* TODO: We need mnemonics */
     if ((xfce_heading = sokoke_xfce_header_new (
         gtk_window_get_icon_name (GTK_WINDOW (dialog)), dialog_title)))
@@ -695,9 +682,7 @@ adblock_check_rule (GRegex*      regex,
             return FALSE;
     }
     /* TODO: Domain opt check */
-    #ifdef G_ENABLE_DEBUG
     adblock_debug ("blocked by pattern regexp=%s -- %s", g_regex_get_pattern (regex), req_uri);
-    #endif
     return TRUE;
 }
 
@@ -1108,6 +1093,7 @@ adblock_app_add_browser_cb (MidoriApp*       app,
 {
     GtkWidget* statusbar;
     GtkWidget* image;
+    GList* children;
     GtkWidget* view;
     gint i;
 
@@ -1119,9 +1105,10 @@ adblock_app_add_browser_cb (MidoriApp*       app,
     g_object_set_data_full (G_OBJECT (browser), "status-image", image,
                             (GDestroyNotify)gtk_widget_destroy);
 
-    i = 0;
-    while((view = midori_browser_get_nth_tab(browser, i++)))
-        adblock_add_tab_cb (browser, MIDORI_VIEW (view), extension);
+    children = midori_browser_get_tabs (MIDORI_BROWSER (browser));
+    for (; children; children = g_list_next (children))
+        adblock_add_tab_cb (browser, children->data, extension);
+    g_list_free (children);
 
     g_signal_connect (browser, "add-tab",
         G_CALLBACK (adblock_add_tab_cb), extension);
@@ -1224,9 +1211,7 @@ adblock_compile_regexp (GString* gpatt,
             if (!g_regex_match_simple ("[\\*]", sig, G_REGEX_UNGREEDY, G_REGEX_MATCH_NOTEMPTY) &&
                 !g_hash_table_lookup (keys, sig))
             {
-                #ifdef G_ENABLE_DEBUG
                 adblock_debug ("sig: %s %s", sig, patt);
-                #endif
                 g_hash_table_insert (keys, sig, regex);
                 g_hash_table_insert (optslist, sig, g_strdup (opts));
                 signature_count++;
@@ -1236,9 +1221,7 @@ adblock_compile_regexp (GString* gpatt,
                 if (g_regex_match_simple ("^\\*", sig, G_REGEX_UNGREEDY, G_REGEX_MATCH_NOTEMPTY) &&
                     !g_hash_table_lookup (pattern, patt))
                 {
-                    #ifdef G_ENABLE_DEBUG
                     adblock_debug ("patt2: %s %s", sig, patt);
-                    #endif
                     g_hash_table_insert (pattern, patt, regex);
                     g_hash_table_insert (optslist, patt, g_strdup (opts));
                 }
@@ -1254,9 +1237,7 @@ adblock_compile_regexp (GString* gpatt,
     }
     else
     {
-        #ifdef G_ENABLE_DEBUG
         adblock_debug ("patt: %s%s", patt, "");
-        #endif
         /* Pattern is a regexp chars */
         g_hash_table_insert (pattern, patt, regex);
         g_hash_table_insert (optslist, patt, g_strdup (opts));
@@ -1311,9 +1292,7 @@ adblock_add_url_pattern (gchar* prefix,
 
     format_patt = adblock_fixup_regexp (prefix, patt);
 
-    #ifdef G_ENABLE_DEBUG
     adblock_debug ("got: %s opts %s", format_patt->str, opts);
-    #endif
     should_free = adblock_compile_regexp (format_patt, opts);
 
     if (data[1] && data[2])
@@ -1595,7 +1574,7 @@ adblock_file_is_up_to_date (gchar* path)
                 found_meta = TRUE;
             }
 
-            if (strncmp ("! Last modified", line, 15) == 0
+            if (strncmp ("! Last mod", line, 10) == 0
             ||  strncmp ("! Updated", line, 9) == 0)
             {
                 gchar** parts = g_strsplit (line, ":", 2);
@@ -1644,22 +1623,39 @@ adblock_file_is_up_to_date (gchar* path)
             gboolean use_dots = FALSE;
 
             /* Common dates are 20 Mar 2012, 20.08.2012 */
-            if (g_strrstr (timestamp, "."))
+            if (strrchr (timestamp, '.'))
             {
                 use_dots = TRUE;
-                parts = g_strsplit (timestamp, ".", 4);
+                /* In case of date like '20.08.2012 12:34'
+                 * we should also nuke the time part */
+                if (strrchr (timestamp, ' '))
+                {
+                    gchar** part = g_strsplit (timestamp, " ", 2);
+                    parts = g_strsplit (part[0], ".", 4);
+                    g_strfreev (part);
+                }
+                else
+                    parts = g_strsplit (timestamp, ".", 4);
             }
             else
                 parts = g_strsplit (timestamp, " ", 4);
 
-            g_date_set_day (mod_date, atoi (parts[0]));
-
             if (use_dots)
                 g_date_set_month (mod_date, atoi (parts[1]));
             else
                 g_date_set_month (mod_date, str_month_name_to_gdate (parts[1]));
 
-            g_date_set_year (mod_date, atoi (parts[2]));
+            /* check if first part is year 201(2) or day */
+            if (strncmp (parts[0], "201", 3) == 0)
+            {
+                g_date_set_day (mod_date, atoi (parts[2]));
+                g_date_set_year (mod_date, atoi (parts[0]));
+            }
+            else
+            {
+                g_date_set_day (mod_date, atoi (parts[0]));
+                g_date_set_year (mod_date, atoi (parts[2]));
+            }
             g_strfreev (parts);
 
             g_date_set_time_t (current, time (NULL));
@@ -1726,7 +1722,7 @@ static void
 adblock_deactivate_cb (MidoriExtension* extension,
                        MidoriBrowser*   browser)
 {
-    gint i;
+    GList* children;
     GtkWidget* view;
     MidoriApp* app = midori_extension_get_app (extension);
     MidoriWebSettings* settings = katze_object_get_object (app, "settings");
@@ -1742,9 +1738,10 @@ adblock_deactivate_cb (MidoriExtension* extension,
     g_signal_handlers_disconnect_by_func (
         browser, adblock_remove_tab_cb, extension);
 
-    i = 0;
-    while((view = midori_browser_get_nth_tab(browser, i++)))
-        adblock_deactivate_tabs (MIDORI_VIEW (view), browser, extension);
+    children = midori_browser_get_tabs (MIDORI_BROWSER (browser));
+    for (; children; children = g_list_next (children))
+        adblock_deactivate_tabs (children->data, browser, extension);
+    g_list_free (children);
 
     adblock_destroy_db ();
     midori_web_settings_remove_style (settings, "adblock-blockcss");
@@ -1769,7 +1766,6 @@ adblock_activate_cb (MidoriExtension* extension,
     g_object_unref (browsers);
 }
 
-#if G_ENABLE_DEBUG
 static void
 test_adblock_parse (void)
 {
@@ -1788,6 +1784,7 @@ test_adblock_parse (void)
 
     g_assert_cmpstr (adblock_parse_line (".*foo/bar"), ==, "..*foo/bar");
     g_assert_cmpstr (adblock_parse_line ("http://bla.blub/*"), ==, "http://bla.blub/");
+    adblock_destroy_db ();
 }
 
 static void
@@ -1796,6 +1793,7 @@ test_subscription_update (void)
     gint temp;
     gchar* filename;
 
+    adblock_init_db ();
     temp = g_file_open_tmp ("midori_adblock_update_test_XXXXXX", &filename, NULL);
     close (temp);
 
@@ -1837,8 +1835,24 @@ test_subscription_update (void)
         -1, NULL);
     g_assert (adblock_file_is_up_to_date (filename));
 
+    g_file_set_contents (filename,
+        "[Adblock Plus 2.0]\n"
+        "! Checksum: S4reE8XaYTtAFxe2RjgBPg\n"
+        "! Last modification time (GMT): 2012.11.05 13:33\n"
+        "! Expires: 5 days (update frequency)\n",
+        -1, NULL);
+    g_assert (!adblock_file_is_up_to_date (filename));
+
+    g_file_set_contents (filename,
+        "[Adblock Plus 2.0]\n"
+        "! Checksum: S4reE8XaYTtAFxe2RjgBPg\n"
+        "! Last modification time (GMT): 2012.11.05 13:33\n",
+        -1, NULL);
+    g_assert (!adblock_file_is_up_to_date (filename));
+
     g_unlink (filename);
     g_free (filename);
+    adblock_destroy_db ();
 }
 
 static void
@@ -1847,6 +1861,7 @@ test_adblock_pattern (void)
     gint temp;
     gchar* filename;
 
+    adblock_init_db ();
     temp = g_file_open_tmp ("midori_adblock_match_test_XXXXXX", &filename, NULL);
 
     /* TODO: Update some tests and add new ones. */
@@ -1895,8 +1910,7 @@ test_adblock_pattern (void)
 
     close (temp);
     g_unlink (filename);
-
-    g_hash_table_destroy (pattern);
+    adblock_destroy_db ();
 }
 
 void
@@ -1906,7 +1920,6 @@ extension_test (void)
     g_test_add_func ("/extensions/adblock/pattern", test_adblock_pattern);
     g_test_add_func ("/extensions/adblock/update", test_subscription_update);
 }
-#endif
 
 MidoriExtension*
 extension_init (void)
