@@ -10,8 +10,8 @@
 */
 
 #include "katze-item.h"
-
 #include "katze-utils.h"
+#include "midori/midori-core.h"
 
 #include <glib/gi18n.h>
 
@@ -384,6 +384,9 @@ katze_item_set_uri (KatzeItem*   item,
 {
     g_return_if_fail (KATZE_IS_ITEM (item));
 
+    if (!g_strcmp0 (item->uri, uri))
+        return;
+
     katze_assign (item->uri, g_strdup (uri));
     g_object_notify (G_OBJECT (item), "uri");
 }
@@ -442,59 +445,96 @@ katze_item_get_pixbuf (KatzeItem* item,
 
     g_return_val_if_fail (KATZE_IS_ITEM (item), NULL);
 
-    if (item->uri == NULL)
-        return NULL;
-
-    #if WEBKIT_CHECK_VERSION (1, 8, 0)
-    /* FIXME: Don't hard-code icon size */
-    if ((pixbuf = webkit_favicon_database_try_get_favicon_pixbuf (
-        webkit_get_favicon_database (), item->uri, 16, 16)))
+    if (widget && KATZE_ITEM_IS_FOLDER (item))
+        return gtk_widget_render_icon (widget, GTK_STOCK_DIRECTORY, GTK_ICON_SIZE_MENU, NULL);
+    if ((pixbuf = midori_paths_get_icon (katze_item_get_icon (item), NULL)))
         return pixbuf;
-    #else
-    if ((pixbuf = g_object_get_data (G_OBJECT (item), "pixbuf")))
+    if ((pixbuf = midori_paths_get_icon (item->uri, widget)))
         return pixbuf;
-    #endif
     return NULL;
 }
 
+static void
+katze_item_image_destroyed_cb (GtkWidget* image,
+                               KatzeItem* item);
+
+#if WEBKIT_CHECK_VERSION (1, 3, 13)
+static void
+#if WEBKIT_CHECK_VERSION (1, 8, 0)
+katze_item_icon_loaded_cb (WebKitFaviconDatabase* database,
+#elif WEBKIT_CHECK_VERSION (1, 3, 13)
+katze_item_icon_loaded_cb (WebKitIconDatabase*    database,
+                           WebKitWebFrame*        web_frame,
+#endif
+                           const gchar*           frame_uri,
+                           GtkWidget*             image)
+{
+    KatzeItem* item = g_object_get_data (G_OBJECT (image), "KatzeItem");
+    GdkPixbuf* pixbuf;
+    if (!strcmp (frame_uri, item->uri)
+      && (pixbuf = midori_paths_get_icon (frame_uri, image)))
+    {
+        gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
+        g_object_unref (pixbuf);
+        /* This signal fires extremely often (WebKit bug?)
+           we must throttle it (disconnect) once we have an icon */
+        katze_item_image_destroyed_cb (image, g_object_ref (item));
+    }
+}
+#endif
+
+static void
+katze_item_image_destroyed_cb (GtkWidget* image,
+                               KatzeItem* item)
+{
+    #if WEBKIT_CHECK_VERSION (1, 8, 0)
+    g_signal_handlers_disconnect_by_func (webkit_get_favicon_database (),
+        katze_item_icon_loaded_cb, image);
+    #elif WEBKIT_CHECK_VERSION (1, 3, 13)
+    g_signal_handlers_disconnect_by_func (webkit_get_icon_database (),
+        katze_item_icon_loaded_cb, image);
+    #endif
+    g_object_unref (item);
+}
+
 /**
  * katze_item_get_image:
  * @item: a #KatzeItem
+ * @widget: a #GtkWidget, or %NULL
  *
  * Retrieves a #GtkImage fit to display @item.
  *
  * Return value: the icon of the item
  *
  * Since: 0.4.4
+ * Since 0.4.8 a @widget was added and the image is visible.
  **/
 GtkWidget*
-katze_item_get_image (KatzeItem* item)
+katze_item_get_image (KatzeItem* item,
+                      GtkWidget* widget)
 {
     GtkWidget* image;
     GdkPixbuf* pixbuf;
-    const gchar* icon;
 
     g_return_val_if_fail (KATZE_IS_ITEM (item), NULL);
 
+    pixbuf = katze_item_get_pixbuf (item, widget);
+    image = gtk_image_new_from_pixbuf (pixbuf);
+    gtk_widget_show (image);
+    if (pixbuf != NULL)
+        g_object_unref (pixbuf);
     if (KATZE_ITEM_IS_FOLDER (item))
-        image = gtk_image_new_from_stock (GTK_STOCK_DIRECTORY, GTK_ICON_SIZE_MENU);
-    /* FIXME: Pass widget for icon size */
-    else if ((pixbuf = katze_item_get_pixbuf (item, NULL)))
-        image = gtk_image_new_from_pixbuf (pixbuf);
-    else if ((icon = katze_item_get_icon (item)) && !strchr (icon, '/'))
-        image = gtk_image_new_from_icon_name (icon, GTK_ICON_SIZE_MENU);
-    else
-    {
-        if (!(icon && (pixbuf = katze_load_cached_icon (icon, NULL))))
-            pixbuf = katze_load_cached_icon (item->uri, NULL);
-        if (pixbuf)
-        {
-            image = gtk_image_new_from_pixbuf (pixbuf);
-            g_object_unref (pixbuf);
-        }
-        else
-            image = gtk_image_new_from_stock (GTK_STOCK_FILE, GTK_ICON_SIZE_MENU);
-    }
+        return image;
+    g_object_set_data (G_OBJECT (image), "KatzeItem", g_object_ref (item));
+    g_signal_connect (image, "destroy",
+        G_CALLBACK (katze_item_image_destroyed_cb), item);
+    #if WEBKIT_CHECK_VERSION (1, 8, 0)
+    g_signal_connect (webkit_get_favicon_database (), "icon-loaded",
+        G_CALLBACK (katze_item_icon_loaded_cb), image);
+    #elif WEBKIT_CHECK_VERSION (1, 3, 13)
+    g_signal_connect (webkit_get_icon_database (), "icon-loaded",
+        G_CALLBACK (katze_item_icon_loaded_cb), image);
+    #endif
     return image;
 }
 
