@@ -22,7 +22,6 @@
 
 #include <glib/gstdio.h>
 #include <libsoup/soup.h>
-#include <webkit/webkit.h>
 
 struct _KatzeNet
 {
@@ -78,6 +77,7 @@ katze_net_priv_free (KatzeNetPriv* priv)
     g_slice_free (KatzeNetPriv, priv);
 }
 
+#if !WEBKIT_CHECK_VERSION (1, 3, 13)
 gchar*
 katze_net_get_cached_path (KatzeNet*    net,
                            const gchar* uri,
@@ -88,6 +88,9 @@ katze_net_get_cached_path (KatzeNet*    net,
     gchar* cached_filename;
     gchar* cached_path;
 
+    if (uri == NULL)
+        return NULL;
+
     checksum = g_compute_checksum_for_string (G_CHECKSUM_MD5, uri, -1);
     extension = g_strrstr (uri, ".");
     cached_filename = g_strdup_printf ("%s%s", checksum,
@@ -96,7 +99,7 @@ katze_net_get_cached_path (KatzeNet*    net,
 
     if (subfolder)
     {
-        gchar* cache_path = g_build_filename (midori_paths_get_cache_dir (), subfolder, NULL);
+        gchar* cache_path = g_build_filename (midori_paths_get_cache_dir_for_reading (), subfolder, NULL);
         katze_mkdir_with_parents (cache_path, 0700);
         cached_path = g_build_filename (cache_path, cached_filename, NULL);
         g_free (cache_path);
@@ -107,6 +110,7 @@ katze_net_get_cached_path (KatzeNet*    net,
     g_free (cached_filename);
     return cached_path;
 }
+#endif
 
 static void
 katze_net_got_body_cb (SoupMessage*  msg,
