@@ -29,10 +29,6 @@
     #include <unistd.h>
 #endif
 
-#ifdef HAVE_HILDON_2_2
-    #include <hildon/hildon.h>
-#endif
-
 #define I_ g_intern_static_string
 
 static void
@@ -42,12 +38,7 @@ proxy_toggle_button_toggled_cb (GtkToggleButton* button,
     gboolean toggled;
     const gchar* property;
 
-    #ifdef HAVE_HILDON_2_2
-    if (HILDON_IS_CHECK_BUTTON (button))
-        toggled = hildon_check_button_get_active (HILDON_CHECK_BUTTON (button));
-    #else
     toggled = gtk_toggle_button_get_active (button);
-    #endif
     property = g_object_get_data (G_OBJECT (button), "property");
     g_object_set (object, property, toggled, NULL);
 }
@@ -219,17 +210,6 @@ proxy_spin_button_changed_cb (GtkSpinButton* button,
     }
 }
 
-#ifdef HAVE_HILDON_2_2
-static void
-proxy_picker_button_changed_cb (HildonPickerButton* button,
-                                GObject*            object)
-{
-    gint value = hildon_picker_button_get_active (button);
-    const gchar* property = g_object_get_data (G_OBJECT (button), "property");
-    g_object_set (object, property, value, NULL);
-    /* FIXME: Implement custom-PROPERTY */
-}
-#else
 static void
 proxy_combo_box_changed_cb (GtkComboBox* button,
                             GObject*     object)
@@ -287,7 +267,6 @@ proxy_combo_box_changed_cb (GtkComboBox* button,
         }
     }
 }
-#endif
 
 static void
 proxy_object_notify_boolean_cb (GObject*    object,
@@ -456,23 +435,14 @@ katze_property_proxy (gpointer     object,
         gchar* notify_property;
         gboolean toggled = katze_object_get_boolean (object, property);
 
-        #ifdef HAVE_HILDON_2_2
-        if (_hint != I_("toggle"))
-        {
-            widget = hildon_check_button_new (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH);
-            gtk_button_set_label (GTK_BUTTON (widget), gettext (nick));
-            hildon_check_button_set_active (HILDON_CHECK_BUTTON (widget), toggled);
-        }
+
+        widget = gtk_check_button_new ();
+        if (_hint == I_("toggle"))
+            gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (widget), FALSE);
         else
-        #endif
-        {
-            widget = gtk_check_button_new ();
-            if (_hint == I_("toggle"))
-                gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (widget), FALSE);
-            else
-                gtk_button_set_label (GTK_BUTTON (widget), gettext (nick));
-            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), toggled);
-        }
+            gtk_button_set_label (GTK_BUTTON (widget), gettext (nick));
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), toggled);
+
         g_signal_connect (widget, "toggled",
                           G_CALLBACK (proxy_toggle_button_toggled_cb), object);
         notify_property = g_strdup_printf ("notify::%s", property);
@@ -740,10 +710,7 @@ katze_property_proxy (gpointer     object,
         widget = gtk_spin_button_new_with_range (
             G_PARAM_SPEC_INT (pspec)->minimum,
             G_PARAM_SPEC_INT (pspec)->maximum, 1);
-        #if HAVE_HILDON
-        hildon_gtk_entry_set_input_mode (GTK_ENTRY (widget),
-                                         HILDON_GTK_INPUT_MODE_NUMERIC);
-        #endif
+
         /* Keep it narrow, 5 digits are usually fine */
         gtk_entry_set_width_chars (GTK_ENTRY (widget), 5);
         gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), value);
@@ -761,39 +728,17 @@ katze_property_proxy (gpointer     object,
         if (hint && g_str_has_prefix (hint, "custom-"))
             custom = &hint[7];
 
-        #ifdef HAVE_HILDON_2_2
-        GtkWidget* selector;
-
-        widget = hildon_picker_button_new (
-            HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH,
-            HILDON_BUTTON_ARRANGEMENT_HORIZONTAL);
-        selector = hildon_touch_selector_new_text ();
-        hildon_button_set_title (HILDON_BUTTON (widget), gettext (nick));
-        hildon_picker_button_set_selector (HILDON_PICKER_BUTTON (widget),
-                                           HILDON_TOUCH_SELECTOR (selector));
-        #else
         widget = gtk_combo_box_text_new ();
-        #endif
         for (i = 0; i < enum_class->n_values; i++)
         {
             const gchar* raw_label = gettext (enum_class->values[i].value_nick);
             gchar* label = katze_strip_mnemonics (raw_label);
-            #ifdef HAVE_HILDON_2_2
-            hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector), label);
-            #else
             gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (widget), label);
-            #endif
             g_free (label);
         }
-        #ifdef HAVE_HILDON_2_2
-        hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (selector), 0, value);
-        g_signal_connect (widget, "value-changed",
-                          G_CALLBACK (proxy_picker_button_changed_cb), object);
-        #else
         gtk_combo_box_set_active (GTK_COMBO_BOX (widget), value);
         g_signal_connect (widget, "changed",
                           G_CALLBACK (proxy_combo_box_changed_cb), object);
-        #endif
         if (custom)
         {
             gchar* custom_text = katze_object_get_string (object, custom);
@@ -1356,70 +1301,17 @@ katze_object_get_object (gpointer     object,
  * Create a directory if it doesn't already exist. Create intermediate
  * parent directories as needed, too.
  *
- * Similar to g_mkdir_with_parents() but returning early if the
- * @pathname refers to an existing directory.
- *
  * Returns: 0 if the directory already exists, or was successfully
  * created. Returns -1 if an error occurred, with errno set.
  *
  * Since: 0.2.1
  */
-/* Creating directories recursively
-   Copyright 2000 Red Hat, Inc.
-   Originally copied from Glib 2.20, coding style adjusted
-   Modified to determine file existence early and pathname must be != NULL */
 int
 katze_mkdir_with_parents (const gchar* pathname,
                           int          mode)
 {
-  gchar* fn, *p;
-
-  /* Use g_access instead of g_file_test for better performance */
-  if (g_access (pathname, F_OK) == 0)
-      return 0;
-
-  fn = g_strdup (pathname);
-
-  if (g_path_is_absolute (fn))
-    p = (gchar *) g_path_skip_root (fn);
-  else
-    p = fn;
-
-  do
-  {
-      while (*p && !G_IS_DIR_SEPARATOR (*p))
-          p++;
-
-      if (!*p)
-          p = NULL;
-      else
-          *p = '\0';
-
-      if (g_access (fn, F_OK) != 0)
-      {
-          if (g_mkdir (fn, mode) == -1)
-          {
-              g_free (fn);
-              return -1;
-          }
-      }
-      else if (!g_file_test (fn, G_FILE_TEST_IS_DIR))
-      {
-          g_free (fn);
-          return -1;
-      }
-      if (p)
-      {
-          *p++ = G_DIR_SEPARATOR;
-          while (*p && G_IS_DIR_SEPARATOR (*p))
-              p++;
-      }
-  }
-  while (p);
-
-  g_free (fn);
-
-  return 0;
+    midori_paths_mkdir_with_parents (pathname, mode);
+    return 0;
 }
 
 /**
@@ -1455,67 +1347,6 @@ katze_widget_has_touchscreen_mode (GtkWidget* widget)
     }
 }
 
-/**
- * katze_load_cached_icon:
- * @uri: an URI string
- * @widget: a #GtkWidget, or %NULL
- *
- * Loads a cached icon for the specified @uri. If there is no
- * icon and @widget is specified, a default will be returned.
- *
- * Returns: a #GdkPixbuf, or %NULL
- *
- * Since: 0.2.2
- */
-GdkPixbuf*
-katze_load_cached_icon (const gchar* uri,
-                        GtkWidget*   widget)
-{
-    GdkPixbuf* icon = NULL;
-
-    g_return_val_if_fail (uri != NULL, NULL);
-
-    if (midori_uri_is_http (uri))
-    {
-        guint i;
-        gchar* icon_uri;
-        gchar* checksum;
-        gchar* ext;
-        gchar* filename;
-        gchar* path;
-
-        i = 8;
-        while (uri[i] != '\0' && uri[i] != '/')
-            i++;
-        if (uri[i] == '/')
-        {
-            gchar* ticon_uri = g_strdup (uri);
-            ticon_uri[i] = '\0';
-            icon_uri = g_strdup_printf ("%s/favicon.ico", ticon_uri);
-            g_free (ticon_uri);
-        }
-        else
-            icon_uri = g_strdup_printf ("%s/favicon.ico", uri);
-
-        checksum = g_compute_checksum_for_string (G_CHECKSUM_MD5, icon_uri, -1);
-        ext = g_strrstr (icon_uri, ".");
-        filename = g_strdup_printf ("%s%s", checksum, ext ? ext : "");
-        g_free (icon_uri);
-        g_free (checksum);
-        path = g_build_filename (midori_paths_get_cache_dir (), "icons", filename, NULL);
-        g_free (filename);
-        if ((icon = gdk_pixbuf_new_from_file_at_size (path, 16, 16, NULL)))
-        {
-            g_free (path);
-            return icon;
-        }
-        g_free (path);
-    }
-
-    return icon || !widget ? icon : gtk_widget_render_icon (widget,
-        GTK_STOCK_FILE, GTK_ICON_SIZE_MENU, NULL);
-}
-
 static void
 katze_uri_entry_changed_cb (GtkWidget* entry,
                             GtkWidget* other_widget)
@@ -1590,6 +1421,9 @@ GtkWidget*
 katze_uri_entry_new (GtkWidget* other_widget)
 {
     GtkWidget* entry = gtk_entry_new ();
+    #if GTK_CHECK_VERSION (3, 6, 0)
+    gtk_entry_set_input_purpose (GTK_ENTRY (entry), GTK_INPUT_PURPOSE_URL);
+    #endif
 
     gtk_entry_set_icon_from_gicon (GTK_ENTRY (entry), GTK_ENTRY_ICON_PRIMARY,
         g_themed_icon_new_with_default_fallbacks ("text-html-symbolic"));
@@ -1636,3 +1470,21 @@ katze_assert_str_equal (const gchar* input,
     }
 }
 
+void
+katze_window_set_sensible_default_size (GtkWindow* window)
+{
+    GdkScreen* screen;
+    GdkRectangle monitor;
+    gint width, height;
+
+    g_return_if_fail (GTK_IS_WINDOW (window));
+
+    screen = gtk_window_get_screen (window);
+    gdk_screen_get_monitor_geometry (screen, 0, &monitor);
+    width = monitor.width / 1.7;
+    height = monitor.height / 1.7;
+    gtk_window_set_default_size (window, width, height);
+    /* 700x100 is the approximate useful minimum dimensions */
+    gtk_widget_set_size_request (GTK_WIDGET (window), 700, 100);
+}
+
