@@ -153,10 +153,6 @@ katze_mkdir_with_parents             (const gchar* pathname,
 gboolean
 katze_widget_has_touchscreen_mode    (GtkWidget*      widget);
 
-GdkPixbuf*
-katze_load_cached_icon               (const gchar*    uri,
-                                      GtkWidget*      widget);
-
 GtkWidget*
 katze_uri_entry_new                  (GtkWidget*      other_widget);
 
@@ -169,6 +165,9 @@ katze_assert_str_equal               (const gchar*    input,
                                       const gchar*    result,
                                       const gchar*    expected);
 
+void
+katze_window_set_sensible_default_size (GtkWindow*    window);
+
 G_END_DECLS
 
 #endif /* __KATZE_UTILS_H__ */
