@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2008-2010 Christian Dywan <christian@twotoasts.de>
+ Copyright (C) 2008-2012 Christian Dywan <christian@twotoasts.de>
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -33,16 +33,7 @@
     #include <locale.h>
 #endif
 
-#if HAVE_HILDON
-    #include <libosso.h>
-    #ifdef HAVE_HILDON_2_2
-        #include <dbus/dbus.h>
-        #include <mce/mode-names.h>
-        #include <mce/dbus-names.h>
-    #endif
-    typedef osso_context_t* MidoriAppInstance;
-    #define MidoriAppInstanceNull NULL
-#elif HAVE_UNIQUE
+#if HAVE_UNIQUE
     typedef gpointer MidoriAppInstance;
     #define MidoriAppInstanceNull NULL
     #if defined(G_DISABLE_DEPRECATED) && !defined(G_CONST_RETURN)
@@ -66,6 +57,10 @@
     #endif
 #endif
 
+#ifdef HAVE_SIGNAL_H
+    #include <signal.h>
+#endif
+
 struct _MidoriApp
 {
     GObject parent_instance;
@@ -82,7 +77,7 @@ struct _MidoriApp
     MidoriBrowser* browser;
     MidoriAppInstance instance;
 
-    #if !HAVE_HILDON || !HAVE_LIBNOTIFY
+    #if !HAVE_LIBNOTIFY
     gchar* program_notify_send;
     #endif
 };
@@ -226,7 +221,6 @@ _midori_app_add_browser (MidoriApp*     app,
         NULL);
     g_signal_connect_swapped (browser, "send-notification",
         G_CALLBACK (midori_app_send_notification), app);
-
     katze_array_add_item (app->browsers, browser);
 
     #if GTK_CHECK_VERSION (3, 0, 0)
@@ -264,10 +258,28 @@ _midori_app_add_browser (MidoriApp*     app,
     #endif
 }
 
+#ifdef HAVE_SIGNAL_H
+static MidoriApp* app_singleton;
+static void
+midori_app_signal_handler (int signal_id)
+{
+    signal (signal_id, 0);
+    if (!midori_paths_is_readonly ())
+        midori_app_quit (app_singleton);
+    if (kill (getpid (), signal_id))
+      exit (1);
+}
+#endif
+
 static void
 _midori_app_quit (MidoriApp* app)
 {
-    gtk_main_quit ();
+    if (!midori_paths_is_readonly ())
+    {
+        gchar* config_file = midori_paths_get_config_filename_for_writing ("running");
+        g_unlink (config_file);
+        g_free (config_file);
+    }
 }
 
 static void
@@ -543,19 +555,10 @@ midori_app_command_received (MidoriApp*   app,
                     else
                     {
                         /* Switch to already open tab if possible */
-                        guint i = 0;
-                        GtkWidget* tab;
-                        gboolean found = FALSE;
-                        while ((tab = midori_browser_get_nth_tab (browser, i++)))
-                            if (g_str_equal (
-                                midori_view_get_display_uri (MIDORI_VIEW (tab)),
-                                fixed_uri))
-                            {
-                                found = TRUE;
-                                break;
-                            }
-                        if (found)
-                            midori_browser_set_current_tab (browser, tab);
+                        KatzeArray* items = midori_browser_get_proxy_array (browser);
+                        KatzeItem* found = katze_array_find_uri (items, fixed_uri);
+                        if (found != NULL)
+                            midori_browser_set_current_item (browser, found);
                         else
                             midori_browser_set_current_page (browser,
                                 midori_browser_add_uri (browser, fixed_uri));
@@ -584,36 +587,7 @@ midori_app_command_received (MidoriApp*   app,
     return FALSE;
 }
 
-#if HAVE_HILDON
-static osso_return_t
-midori_app_osso_rpc_handler_cb (const gchar* interface,
-                                const gchar* method,
-                                GArray*      arguments,
-                                gpointer     data,
-                                osso_rpc_t * retval)
-{
-    MidoriApp* app = MIDORI_APP (data);
-    GdkScreen* screen = NULL;
-    gboolean success;
-
-    if (!g_strcmp0 (method, "top_application"))
-        success = midori_app_command_received (app, "activate", NULL, screen);
-    else if (!g_strcmp0 (method, "new"))
-        success = midori_app_command_received (app, "new", NULL, screen);
-    else if (!g_strcmp0 (method, "open"))
-    {
-        /* FIXME: Handle arguments */
-        success = midori_app_command_received (app, "open", NULL, screen);
-    }
-    else if (!g_strcmp0 (method, "command"))
-    {
-        /* FIXME: Handle arguments */
-        success = midori_app_command_received (app, "command", NULL, screen);
-    }
-
-    return success ? OSSO_OK : OSSO_INVALID;
-}
-#elif HAVE_UNIQUE
+#if HAVE_UNIQUE
 static UniqueResponse
 midori_browser_message_received_cb (UniqueApp*         instance,
                                     gint               command,
@@ -713,34 +687,9 @@ static MidoriAppInstance
 midori_app_create_instance (MidoriApp* app)
 {
     MidoriAppInstance instance;
-
-    #if HAVE_HILDON
-    instance = osso_initialize (PACKAGE_NAME, PACKAGE_VERSION, FALSE, NULL);
-
-    if (!instance)
-    {
-        g_critical ("Error initializing OSSO D-Bus context - Midori");
-        return NULL;
-    }
-
-    if (osso_rpc_set_default_cb_f (instance, midori_app_osso_rpc_handler_cb,
-                                   app) != OSSO_OK)
-    {
-        g_critical ("Error initializing remote procedure call handler - Midori");
-        osso_deinitialize (instance);
-        return NULL;
-    }
-
-    #ifdef HAVE_HILDON_2_2
-    if (OSSO_OK == osso_rpc_run_system (instance, MCE_SERVICE, MCE_REQUEST_PATH,
-        MCE_REQUEST_IF, MCE_ACCELEROMETER_ENABLE_REQ, NULL, DBUS_TYPE_INVALID))
-        /* Accelerometer enabled */;
-    #endif
-    #else
     GdkDisplay* display;
     gchar* display_name;
     gchar* instance_name;
-    guint i, n;
     #if !HAVE_UNIQUE
     gboolean exists;
     GIOChannel* channel;
@@ -751,15 +700,17 @@ midori_app_create_instance (MidoriApp* app)
 
     {
         #if HAVE_UNIQUE
-        const gchar* config = midori_paths_get_config_dir ();
-        gchar* name_hash;
-        name_hash = g_compute_checksum_for_string (G_CHECKSUM_MD5, config, -1);
-        katze_assign (app_name, g_strconcat ("midori", "_", name_hash, NULL));
+        const gchar* config = midori_paths_get_config_dir_for_reading ();
+        gchar* config_hash = g_compute_checksum_for_string (G_CHECKSUM_MD5, config, -1);
+        gchar* name_hash = g_compute_checksum_for_string (G_CHECKSUM_MD5, app_name, -1);
+        katze_assign (app_name, g_strconcat (PACKAGE_NAME,
+            "_", config_hash, "_", name_hash, NULL));
+        g_free (config_hash);
         g_free (name_hash);
-        g_object_notify (G_OBJECT (app), "name");
         #else
         katze_assign (app_name, g_strdup (PACKAGE_NAME));
         #endif
+        g_object_notify (G_OBJECT (app), "name");
     }
 
     #ifdef GDK_WINDOWING_X11
@@ -768,10 +719,7 @@ midori_app_create_instance (MidoriApp* app)
     #else
     display_name = g_strdup (gdk_display_get_name (display));
     #endif
-    n = strlen (display_name);
-    for (i = 0; i < n; i++)
-        if (strchr (":.\\/", display_name[i]))
-            display_name[i] = '_';
+    g_strdelimit (display_name, ":.\\/", '_');
     instance_name = g_strdup_printf ("de.twotoasts.%s_%s", app_name, display_name);
     g_free (display_name);
     katze_assign (app_name, instance_name);
@@ -782,7 +730,7 @@ midori_app_create_instance (MidoriApp* app)
     g_signal_connect (instance, "message-received",
                       G_CALLBACK (midori_browser_message_received_cb), app);
     #else
-    instance = socket_init (instance_name, midori_paths_get_config_dir (), &exists);
+    instance = socket_init (instance_name, midori_paths_get_config_dir_for_writing (), &exists);
     g_object_set_data (G_OBJECT (app), "sock-exists",
         exists ? (gpointer)0xdeadbeef : NULL);
     if (instance != MidoriAppInstanceNull)
@@ -792,7 +740,6 @@ midori_app_create_instance (MidoriApp* app)
             (GIOFunc)midori_app_io_channel_watch_cb, app);
     }
     #endif
-    #endif
     return instance;
 }
 
@@ -802,9 +749,44 @@ midori_app_get_name (MidoriApp* app)
     return app_name;
 }
 
+gboolean
+midori_app_get_crashed (MidoriApp* app)
+{
+    if (!midori_paths_is_readonly ())
+    {
+        /* We test for the presence of a dummy file which is created once
+           and deleted during normal runtime, but persists in case of a crash. */
+        gchar* config_file = midori_paths_get_config_filename_for_writing ("running");
+        gboolean crashed = (g_access (config_file, F_OK) == 0);
+        if (!crashed)
+            g_file_set_contents (config_file, "RUNNING", -1, NULL);
+        g_free (config_file);
+        if (crashed)
+            return TRUE;
+    }
+
+    return FALSE;
+}
+
 static void
 midori_app_init (MidoriApp* app)
 {
+    #ifdef HAVE_SIGNAL_H
+    app_singleton = app;
+    #ifdef SIGHUP
+    signal (SIGHUP, &midori_app_signal_handler);
+    #endif
+    #ifdef SIGINT
+    signal (SIGINT, &midori_app_signal_handler);
+    #endif
+    #ifdef SIGTERM
+    signal (SIGTERM, &midori_app_signal_handler);
+    #endif
+    #ifdef SIGQUIT
+    signal (SIGQUIT, &midori_app_signal_handler);
+    #endif
+    #endif
+
     app->settings = NULL;
     app->bookmarks = NULL;
     app->trash = NULL;
@@ -821,7 +803,6 @@ midori_app_init (MidoriApp* app)
     #else
     app->program_notify_send = g_find_program_in_path ("notify-send");
     #endif
-
 }
 
 static void
@@ -839,10 +820,7 @@ midori_app_finalize (GObject* object)
     katze_object_assign (app->extensions, NULL);
     katze_object_assign (app->browsers, NULL);
 
-    #if HAVE_HILDON
-    osso_deinitialize (app->instance);
-    app->instance = NULL;
-    #elif HAVE_UNIQUE
+    #if HAVE_UNIQUE
     katze_object_assign (app->instance, NULL);
     #else
     sock_cleanup ();
@@ -952,12 +930,19 @@ midori_app_get_property (GObject*    object,
  * Return value: a new #MidoriApp
  **/
 MidoriApp*
-midori_app_new (void)
+midori_app_new (const gchar* name)
 {
-    MidoriApp* app = g_object_new (MIDORI_TYPE_APP,
-                                   NULL);
+    return g_object_new (MIDORI_TYPE_APP, "name", name, NULL);
+}
 
-    return app;
+static gboolean instance_is_not_running = FALSE;
+static gboolean instance_is_running = FALSE;
+
+void
+midori_app_set_instance_is_running (gboolean is_running)
+{
+    instance_is_not_running = !is_running;
+    instance_is_running = is_running;
 }
 
 /**
@@ -977,20 +962,19 @@ midori_app_instance_is_running (MidoriApp* app)
 {
     g_return_val_if_fail (MIDORI_IS_APP (app), FALSE);
 
+    if (instance_is_not_running)
+        return FALSE;
+    else if (instance_is_running)
+        return TRUE;
+
     if (app->instance == MidoriAppInstanceNull)
         app->instance = midori_app_create_instance (app);
 
-    #if HAVE_HILDON
-    /* FIXME: Determine if application is running already */
-    if (app->instance)
-        return FALSE;
-    #elif HAVE_UNIQUE
-    if (app->instance)
-        return unique_app_is_running (app->instance);
+    #if HAVE_UNIQUE
+    return app->instance && unique_app_is_running (app->instance);
     #else
-        return g_object_get_data (G_OBJECT (app), "sock-exists") != NULL;
+    return g_object_get_data (G_OBJECT (app), "sock-exists") != NULL;
     #endif
-    return FALSE;
 }
 
 /**
@@ -1007,19 +991,13 @@ midori_app_instance_is_running (MidoriApp* app)
 gboolean
 midori_app_instance_send_activate (MidoriApp* app)
 {
-    #if HAVE_UNIQUE
-    UniqueResponse response;
-    #endif
-
-    /* g_return_val_if_fail (MIDORI_IS_APP (app), FALSE); */
+    g_return_val_if_fail (MIDORI_IS_APP (app), FALSE);
     g_return_val_if_fail (midori_app_instance_is_running (app), FALSE);
 
-    #if HAVE_HILDON
-    osso_application_top (app->instance, PACKAGE_NAME, NULL);
-    #elif HAVE_UNIQUE
+    #if HAVE_UNIQUE
     if (app->instance)
     {
-        response = unique_app_send_message (app->instance, UNIQUE_ACTIVATE, NULL);
+        UniqueResponse response = unique_app_send_message (app->instance, UNIQUE_ACTIVATE, NULL);
         if (response == UNIQUE_RESPONSE_OK)
             return TRUE;
     }
@@ -1045,19 +1023,13 @@ midori_app_instance_send_activate (MidoriApp* app)
 gboolean
 midori_app_instance_send_new_browser (MidoriApp* app)
 {
-    #if HAVE_UNIQUE
-    UniqueResponse response;
-    #endif
-
-    /* g_return_val_if_fail (MIDORI_IS_APP (app), FALSE); */
+    g_return_val_if_fail (MIDORI_IS_APP (app), FALSE);
     g_return_val_if_fail (midori_app_instance_is_running (app), FALSE);
 
-    #if HAVE_HILDON
-    osso_application_top (app->instance, PACKAGE_NAME, "new");
-    #elif HAVE_UNIQUE
+    #if HAVE_UNIQUE
     if (app->instance)
     {
-        response = unique_app_send_message (app->instance, UNIQUE_NEW, NULL);
+        UniqueResponse response = unique_app_send_message (app->instance, UNIQUE_NEW, NULL);
         if (response == UNIQUE_RESPONSE_OK)
             return TRUE;
     }
@@ -1087,20 +1059,26 @@ gboolean
 midori_app_instance_send_uris (MidoriApp* app,
                                gchar**    uris)
 {
-    #if HAVE_UNIQUE
-    UniqueMessageData* message;
-    UniqueResponse response;
-    #endif
-
-    /* g_return_val_if_fail (MIDORI_IS_APP (app), FALSE); */
+    g_return_val_if_fail (MIDORI_IS_APP (app), FALSE);
     g_return_val_if_fail (midori_app_instance_is_running (app), FALSE);
     g_return_val_if_fail (uris != NULL, FALSE);
 
-    #if HAVE_HILDON
-    /* FIXME: Implement */
-    #elif HAVE_UNIQUE
+    #if HAVE_UNIQUE
     if (app->instance)
     {
+        UniqueMessageData* message;
+        UniqueResponse response;
+        /* Encode any IDN addresses because libUnique doesn't like them */
+        int i = 0;
+        while (uris[i] != NULL)
+        {
+            gchar* new_uri = sokoke_prepare_uri (uris[i]);
+            gchar* escaped_uri = g_uri_escape_string (new_uri, NULL, FALSE);
+            g_free (new_uri);
+            katze_assign (uris[i], escaped_uri);
+            i++;
+        }
+
         message = unique_message_data_new ();
         unique_message_data_set_uris (message, uris);
         response = unique_app_send_message (app->instance, UNIQUE_OPEN, message);
@@ -1137,12 +1115,7 @@ gboolean
 midori_app_send_command (MidoriApp* app,
                          gchar**    command)
 {
-    #if HAVE_UNIQUE
-    UniqueMessageData* message;
-    UniqueResponse response;
-    #endif
-
-    /* g_return_val_if_fail (MIDORI_IS_APP (app), FALSE); */
+    g_return_val_if_fail (MIDORI_IS_APP (app), FALSE);
     g_return_val_if_fail (command != NULL, FALSE);
 
     if (midori_app_instance_is_running (app))
@@ -1160,12 +1133,11 @@ midori_app_send_command (MidoriApp* app,
     else
         return midori_app_command_received (app, "command", command, NULL);
 
-    #if HAVE_HILDON
-    /* FIXME: Implement */
-    #elif HAVE_UNIQUE
+    #if HAVE_UNIQUE
     if (app->instance)
     {
-        message = unique_message_data_new ();
+        UniqueResponse response;
+        UniqueMessageData* message = unique_message_data_new ();
         unique_message_data_set_uris (message, command);
         response = unique_app_send_message (app->instance,
             MIDORI_UNIQUE_COMMAND, message);
@@ -1206,6 +1178,17 @@ midori_app_add_browser (MidoriApp*     app,
     g_signal_emit (app, signals[ADD_BROWSER], 0, browser);
 }
 
+void
+midori_app_set_browsers (MidoriApp*     app,
+                         KatzeArray*    browsers,
+                         MidoriBrowser* browser)
+{
+    g_return_if_fail (MIDORI_IS_APP (app));
+    g_return_if_fail (KATZE_IS_ARRAY (browsers));
+    katze_object_assign (app->browsers, g_object_ref (browsers));
+    app->browser = browser;
+}
+
 /**
  * midori_app_create_browser:
  * @app: a #MidoriApp
@@ -1309,17 +1292,13 @@ midori_app_send_notification (MidoriApp*   app,
     g_return_if_fail (MIDORI_IS_APP (app));
     g_return_if_fail (title);
 
-    #if HAVE_HILDON
-    hildon_banner_show_information_with_markup (GTK_WIDGET (app->browser),
-                                                "midori", message);
-    #elif HAVE_LIBNOTIFY
+    #if HAVE_LIBNOTIFY
     if (notify_is_initted ())
     {
-        NotifyNotification* note;
         #if NOTIFY_CHECK_VERSION (0, 7, 0)
-        note = notify_notification_new (title, message, "midori");
+        NotifyNotification* note = notify_notification_new (title, message, "midori");
         #else
-        note = notify_notification_new (title, message, "midori", NULL);
+        NotifyNotification* note = notify_notification_new (title, message, "midori", NULL);
         #endif
         notify_notification_show (note, NULL);
         g_object_unref (note);
@@ -1350,16 +1329,16 @@ midori_app_send_notification (MidoriApp*   app,
  *
  * Since: 0.4.2
  **/
-gboolean
+void
 midori_app_setup (gint               *argc,
                   gchar**            *argument_vector,
-                  const GOptionEntry *entries,
-                  GError*            *error)
+                  const GOptionEntry *entries)
 {
     GtkIconSource* icon_source;
     GtkIconSet* icon_set;
     GtkIconFactory* factory;
     gsize i;
+    GError* error = NULL;
     gboolean success;
 
     static GtkStockItem items[] =
@@ -1377,7 +1356,7 @@ midori_app_setup (gint               *argc,
         { STOCK_STYLE,        N_("User_styles") },
         { STOCK_TAB_NEW,      N_("New _Tab") },
         { STOCK_TRANSFER,     N_("_Transfers"), GDK_CONTROL_MASK | GDK_SHIFT_MASK, GDK_KEY_J },
-        { STOCK_PLUGINS,      N_("Netscape p_lugins") },
+        { MIDORI_STOCK_PLUGINS,      N_("Netscape p_lugins") },
         { STOCK_USER_TRASH,   N_("_Closed Tabs") },
         { STOCK_WINDOW_NEW,   N_("New _Window") },
         { STOCK_FOLDER_NEW,   N_("New _Folder") },
@@ -1434,8 +1413,16 @@ midori_app_setup (gint               *argc,
     textdomain (GETTEXT_PACKAGE);
     #endif
 
+    #ifdef HAVE_GRANITE_CLUTTER
+    success = gtk_clutter_init_with_args (argc, argument_vector, _("[Addresses]"),
+                                          (GOptionEntry*)entries, GETTEXT_PACKAGE, &error);
+    #elif GTK_CHECK_VERSION (3, 0, 0)
     success = gtk_init_with_args (argc, argument_vector, _("[Addresses]"),
-                                  entries, GETTEXT_PACKAGE, error);
+                                  entries, GETTEXT_PACKAGE, &error);
+    #else
+    success = gtk_init_with_args (argc, argument_vector, _("[Addresses]"),
+                                  (GOptionEntry*)entries, GETTEXT_PACKAGE, &error);
+    #endif
 
     factory = gtk_icon_factory_new ();
     for (i = 0; i < G_N_ELEMENTS (items); i++)
@@ -1452,7 +1439,8 @@ midori_app_setup (gint               *argc,
     gtk_icon_factory_add_default (factory);
     g_object_unref (factory);
 
-    return success;
+    if (!success)
+        g_error ("%s - %s", _("Midori"), error->message);
 }
 
 gboolean
@@ -1460,8 +1448,8 @@ midori_debug (const gchar* token)
 {
     static const gchar* debug_token = NULL;
     const gchar* debug = g_getenv ("MIDORI_DEBUG");
-    const gchar* debug_tokens = "headers body referer cookies paths hsts unarmed ";
-    const gchar* full_debug_tokens = "adblock:match adblock:time startup bookmarks ";
+    const gchar* debug_tokens = "headers body referer cookies paths hsts unarmed bookmarks ";
+    const gchar* full_debug_tokens = "adblock:match adblock:time startup ";
     if (debug_token == NULL)
     {
         gchar* found_token;
