@@ -13,9 +13,6 @@
 #define __MIDORI_BROWSER_H__
 
 #include <webkit/webkit.h>
-#if defined(HAVE_HILDON) && HAVE_HILDON
-    #include <hildon/hildon.h>
-#endif
 
 #include <katze/katze.h>
 #include "midori-view.h"
@@ -40,11 +37,7 @@ typedef struct _MidoriBrowserClass           MidoriBrowserClass;
 
 struct _MidoriBrowserClass
 {
-    #if defined(HAVE_HILDON) && HAVE_HILDON
-    HildonWindowClass parent_class;
-    #else
     GtkWindowClass parent_class;
-    #endif
 
     /* Signals */
     void
@@ -86,14 +79,9 @@ midori_browser_add_tab                (MidoriBrowser*     browser,
                                        GtkWidget*         widget);
 
 void
-midori_browser_remove_tab             (MidoriBrowser*     browser,
+midori_browser_close_tab              (MidoriBrowser*     browser,
                                        GtkWidget*         widget);
 
-void
-midori_browser_foreach                (MidoriBrowser*     browser,
-                                       GtkCallback        callback,
-                                       gpointer           callback_data);
-
 gint
 midori_browser_add_item               (MidoriBrowser*     browser,
                                        KatzeItem*         item);
@@ -139,6 +127,10 @@ midori_browser_set_current_page       (MidoriBrowser*     browser,
 gint
 midori_browser_get_current_page       (MidoriBrowser*     browser);
 
+void
+midori_browser_set_current_item       (MidoriBrowser* browser,
+                                       KatzeItem*     item);
+
 GtkWidget*
 midori_browser_get_nth_tab            (MidoriBrowser*     browser,
                                        gint               n);
@@ -162,9 +154,6 @@ midori_browser_get_tabs               (MidoriBrowser*     browser);
 gint
 midori_browser_get_n_pages            (MidoriBrowser*     browser);
 
-KatzeArray*
-midori_browser_get_proxy_items        (MidoriBrowser*     browser);
-
 KatzeArray*
 midori_browser_get_proxy_array        (MidoriBrowser*     browser);
 
@@ -185,6 +174,15 @@ midori_browser_update_history         (KatzeItem*         item,
                                        const gchar*       type,
                                        const gchar*       event);
 
+void
+midori_browser_save_uri               (MidoriBrowser*     browser,
+                                       MidoriView*        view,
+                                       const gchar*       uri);
+
+void
+midori_browser_set_inactivity_reset   (MidoriBrowser*     browser,
+                                       gint               inactivity_reset);
+
 G_END_DECLS
 
 #endif /* __MIDORI_BROWSER_H__ */
