@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2008-2009 Christian Dywan <christian@twotoasts.de>
+ Copyright (C) 2008-2012 Christian Dywan <christian@twotoasts.de>
  Copyright (C) 2009 Dale Whittaker <dayul@users.sf.net>
 
  This library is free software; you can redistribute it and/or
@@ -21,8 +21,10 @@ G_DEFINE_TYPE (MidoriExtension, midori_extension, G_TYPE_OBJECT);
 
 struct _MidoriExtensionPrivate
 {
+    gchar* stock_id;
     gchar* name;
     gchar* description;
+    gboolean use_markup;
     gchar* version;
     gchar* authors;
     gchar* website;
@@ -125,8 +127,10 @@ enum
 {
     PROP_0,
 
+    PROP_STOCK_ID,
     PROP_NAME,
     PROP_DESCRIPTION,
+    PROP_USE_MARKUP,
     PROP_VERSION,
     PROP_AUTHORS,
     PROP_WEBSITE,
@@ -211,6 +215,15 @@ midori_extension_class_init (MidoriExtensionClass* class)
 
     flags = G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS;
 
+    g_object_class_install_property (gobject_class,
+                                     PROP_STOCK_ID,
+                                     g_param_spec_string (
+                                     "stock-id",
+                                     "Stock ID",
+                                     "An optional icon stock ID",
+                                     NULL,
+                                     flags));
+
     g_object_class_install_property (gobject_class,
                                      PROP_NAME,
                                      g_param_spec_string (
@@ -229,6 +242,15 @@ midori_extension_class_init (MidoriExtensionClass* class)
                                      NULL,
                                      flags));
 
+    g_object_class_install_property (gobject_class,
+                                     PROP_USE_MARKUP,
+                                     g_param_spec_boolean (
+                                     "use-markup",
+                                     "Use Markup",
+                                     "Whether to use Pango markup",
+                                     FALSE,
+                                     flags));
+
     g_object_class_install_property (gobject_class,
                                      PROP_VERSION,
                                      g_param_spec_string (
@@ -310,19 +332,8 @@ midori_extension_activate_cb (MidoriExtension* extension,
             if (error->code == G_FILE_ERROR_NOENT)
             {
                 gchar* filename = g_object_get_data (G_OBJECT (extension), "filename");
-                gchar* folder;
-                if (g_str_has_prefix (filename, MIDORI_MODULE_PREFIX))
-                    filename = &filename[strlen (MIDORI_MODULE_PREFIX)];
-                if (g_str_has_suffix (filename, G_MODULE_SUFFIX))
-                    filename = g_strndup (filename,
-                        strlen (filename) - strlen ("." G_MODULE_SUFFIX));
-                else
-                    filename = g_strdup (filename);
-                folder = g_build_filename ("extensions", filename, NULL);
-                g_free (filename);
                 katze_assign (config_file,
-                    midori_paths_get_preset_filename (folder, "config"));
-                g_free (folder);
+                    midori_paths_get_extension_preset_filename (filename, "config"));
                 g_key_file_load_from_file (extension->priv->key_file, config_file,
                                            G_KEY_FILE_KEEP_COMMENTS, NULL);
             }
@@ -418,6 +429,7 @@ midori_extension_finalize (GObject* object)
     MidoriExtension* extension = MIDORI_EXTENSION (object);
 
     katze_object_assign (extension->priv->app, NULL);
+    katze_assign (extension->priv->stock_id, NULL);
     katze_assign (extension->priv->name, NULL);
     katze_assign (extension->priv->description, NULL);
     katze_assign (extension->priv->version, NULL);
@@ -442,12 +454,18 @@ midori_extension_set_property (GObject*      object,
 
     switch (prop_id)
     {
+    case PROP_STOCK_ID:
+        katze_assign (extension->priv->stock_id, g_value_dup_string (value));
+        break;
     case PROP_NAME:
         katze_assign (extension->priv->name, g_value_dup_string (value));
         break;
     case PROP_DESCRIPTION:
         katze_assign (extension->priv->description, g_value_dup_string (value));
         break;
+    case PROP_USE_MARKUP:
+        extension->priv->use_markup = g_value_get_boolean (value);
+        break;
     case PROP_VERSION:
     {
         /* Don't show version suffix if it matches the running Midori */
@@ -489,12 +507,18 @@ midori_extension_get_property (GObject*    object,
 
     switch (prop_id)
     {
+    case PROP_STOCK_ID:
+        g_value_set_string (value, extension->priv->stock_id);
+        break;
     case PROP_NAME:
         g_value_set_string (value, extension->priv->name);
         break;
     case PROP_DESCRIPTION:
         g_value_set_string (value, extension->priv->description);
         break;
+    case PROP_USE_MARKUP:
+        g_value_set_boolean (value, extension->priv->use_markup);
+        break;
     case PROP_VERSION:
         g_value_set_string (value, extension->priv->version);
         break;
@@ -513,6 +537,163 @@ midori_extension_get_property (GObject*    object,
     }
 }
 
+void
+midori_extension_load_from_folder (MidoriApp* app,
+                                   gchar**    keys,
+                                   gboolean   activate)
+{
+    if (!g_module_supported ())
+        return;
+
+    gchar* extension_path = midori_paths_get_lib_path (PACKAGE_NAME);
+    if (!extension_path)
+        return;
+
+    if (activate)
+    {
+        gint i = 0;
+        const gchar* filename;
+        while (keys && (filename = keys[i++]))
+            midori_extension_activate_gracefully (app, extension_path, filename, activate);
+    }
+    else
+    {
+        GDir* extension_dir = g_dir_open (extension_path, 0, NULL);
+        g_return_if_fail (extension_dir != NULL);
+        const gchar* filename;
+        while ((filename = g_dir_read_name (extension_dir)))
+            midori_extension_activate_gracefully (app, extension_path, filename, activate);
+        g_dir_close (extension_dir);
+    }
+
+    g_free (extension_path);
+}
+
+GObject*
+midori_extension_load_from_file (const gchar* extension_path,
+                                 const gchar* filename,
+                                 gboolean     activate,
+                                 gboolean     test)
+{
+    gchar* fullname;
+    GModule* module;
+    typedef GObject* (*extension_init_func)(void);
+    extension_init_func extension_init;
+    static GHashTable* modules = NULL;
+
+    /* Ignore files which don't have the correct suffix */
+    if (!g_str_has_suffix (filename, G_MODULE_SUFFIX))
+        return NULL;
+
+    if (strchr (filename, '/'))
+    {
+        gchar* clean = g_strndup (filename, strchr (filename, '/') - filename);
+        fullname = g_build_filename (extension_path, clean, NULL);
+        g_free (clean);
+    }
+    else
+        fullname = g_build_filename (extension_path, filename, NULL);
+
+    module = g_module_open (fullname, G_MODULE_BIND_LOCAL);
+    g_free (fullname);
+
+    /* GModule detects repeated loading but exposes no API to check it.
+       Skip any modules that were loaded before. */
+    if (modules == NULL)
+        modules = g_hash_table_new (g_direct_hash, g_direct_equal);
+    if (g_hash_table_lookup (modules, module))
+        return NULL;
+
+    g_hash_table_insert (modules, module, g_strdup (filename));
+
+    if (module && g_module_symbol (module, "extension_init",
+                                   (gpointer) &extension_init))
+    {
+        typedef void (*extension_test_func)(void);
+        extension_test_func extension_test;
+        GObject* extension = extension_init ();
+        if (test && extension && g_module_symbol (module, "extension_test", (gpointer) &extension_test))
+            extension_test ();
+        return extension;
+    }
+
+    return NULL;
+}
+
+void
+midori_extension_activate_gracefully (MidoriApp*   app,
+                                      const gchar* extension_path,
+                                      const gchar* filename,
+                                      gboolean     activate)
+{
+    GObject* extension = midori_extension_load_from_file (extension_path, filename, activate, FALSE);
+
+    midori_extension_activate (extension, filename, activate, app);
+    if (!extension && g_module_error () != NULL)
+    {
+        KatzeArray* extensions = katze_object_get_object (app, "extensions");
+        extension = g_object_new (MIDORI_TYPE_EXTENSION,
+                                  "name", filename,
+                                  "description", g_module_error (),
+                                  NULL);
+        g_warning ("%s", g_module_error ());
+        katze_array_add_item (extensions, extension);
+        g_object_unref (extensions);
+    }
+
+    if (extension != NULL)
+        g_object_unref (extension);
+}
+
+static void
+midori_load_extension (MidoriApp*       app,
+                       MidoriExtension* extension,
+                       const gchar*     filename)
+{
+    KatzeArray* extensions = katze_object_get_object (app, "extensions");
+    /* Signal that we want the extension to load and save */
+    g_object_set_data_full (G_OBJECT (extension), "filename",
+                            g_strdup (filename), g_free);
+    if (midori_extension_is_prepared (extension))
+        midori_extension_get_config_dir (extension);
+    katze_array_add_item (extensions, extension);
+    g_object_unref (extensions);
+}
+
+void
+midori_extension_activate (GObject*     extension,
+                           const gchar* filename,
+                           gboolean     activate,
+                           MidoriApp*   app)
+{
+    if (MIDORI_IS_EXTENSION (extension))
+    {
+        if (filename != NULL)
+            midori_load_extension (app, MIDORI_EXTENSION (extension), filename);
+        if (activate)
+            g_signal_emit_by_name (extension, "activate", app);
+    }
+    else if (KATZE_IS_ARRAY (extension))
+    {
+        MidoriExtension* extension_item;
+        KATZE_ARRAY_FOREACH_ITEM (extension_item, KATZE_ARRAY (extension))
+            if (MIDORI_IS_EXTENSION (extension_item))
+            {
+                gchar* key;
+
+                if (filename != NULL)
+                    midori_load_extension (app, extension_item, filename);
+                if (activate)
+                {
+                    key = katze_object_get_string (extension_item, "key");
+                    if (key && filename && strstr (filename, key))
+                        g_signal_emit_by_name (extension_item, "activate", app);
+                    g_free (key);
+                }
+            }
+    }
+}
+
 /**
  * midori_extension_is_prepared:
  * @extension: a #MidoriExtension
@@ -650,8 +831,7 @@ midori_extension_get_config_dir (MidoriExtension* extension)
     {
         gchar* filename = g_object_get_data (G_OBJECT (extension), "filename");
         if (filename != NULL)
-            extension->priv->config_dir = g_build_filename (
-                midori_paths_get_config_dir (), "extensions", filename, NULL);
+            extension->priv->config_dir = midori_paths_get_extension_config_dir (filename);
         else
             extension->priv->config_dir = NULL;
     }
@@ -687,6 +867,22 @@ midori_extension_install_boolean (MidoriExtension* extension,
                         default_value, FALSE);
 }
 
+static void
+midori_extension_save_settings (MidoriExtension *extension)
+{
+    GError* error = NULL;
+    gchar* config_file = g_build_filename (extension->priv->config_dir, "config", NULL);
+    katze_mkdir_with_parents (extension->priv->config_dir, 0700);
+    sokoke_key_file_save_to_file (extension->priv->key_file, config_file, &error);
+    if (error)
+    {
+        printf (_("The configuration of the extension '%s' couldn't be saved: %s\n"),
+                extension->priv->name, error->message);
+        g_error_free (error);
+    }
+    g_free (config_file);
+}
+
 /**
  * midori_extension_get_boolean:
  * @extension: a #MidoriExtension
@@ -739,23 +935,14 @@ midori_extension_set_boolean (MidoriExtension* extension,
     setting->value = value;
     if (extension->priv->key_file)
     {
-        GError* error = NULL;
-        /* FIXME: Handle readonly folder/ file */
-        gchar* config_file = g_build_filename (extension->priv->config_dir,
-                                               "config", NULL);
-        katze_mkdir_with_parents (extension->priv->config_dir, 0700);
         g_key_file_set_boolean (extension->priv->key_file,
                                 "settings", name, value);
-        sokoke_key_file_save_to_file (extension->priv->key_file, config_file, &error);
-        if (error)
-        {
-            printf (_("The configuration of the extension '%s' couldn't be saved: %s\n"),
-                    extension->priv->name, error->message);
-            g_error_free (error);
-        }
+        midori_extension_save_settings (extension);
     }
 }
 
+
+
 /**
  * midori_extension_install_integer:
  * @extension: a #MidoriExtension
@@ -836,20 +1023,9 @@ midori_extension_set_integer (MidoriExtension* extension,
     setting->value = value;
     if (extension->priv->key_file)
     {
-        GError* error = NULL;
-        /* FIXME: Handle readonly folder/ file */
-        gchar* config_file = g_build_filename (extension->priv->config_dir,
-                                               "config", NULL);
-        katze_mkdir_with_parents (extension->priv->config_dir, 0700);
         g_key_file_set_integer (extension->priv->key_file,
                                 "settings", name, value);
-        sokoke_key_file_save_to_file (extension->priv->key_file, config_file, &error);
-        if (error)
-        {
-            printf (_("The configuration of the extension '%s' couldn't be saved: %s\n"),
-                    extension->priv->name, error->message);
-            g_error_free (error);
-        }
+        midori_extension_save_settings (extension);
     }
 }
 
@@ -933,20 +1109,9 @@ midori_extension_set_string (MidoriExtension* extension,
     katze_assign (setting->value, g_strdup (value));
     if (extension->priv->key_file)
     {
-        GError* error = NULL;
-        /* FIXME: Handle readonly folder/ file */
-        gchar* config_file = g_build_filename (extension->priv->config_dir,
-                                               "config", NULL);
-        katze_mkdir_with_parents (extension->priv->config_dir, 0700);
         g_key_file_set_string (extension->priv->key_file,
                                 "settings", name, value);
-        sokoke_key_file_save_to_file (extension->priv->key_file, config_file, &error);
-        if (error)
-        {
-            printf (_("The configuration of the extension '%s' couldn't be saved: %s\n"),
-                    extension->priv->name, error->message);
-            g_error_free (error);
-        }
+        midori_extension_save_settings (extension);
     }
 }
 
@@ -1045,19 +1210,8 @@ midori_extension_set_string_list (MidoriExtension* extension,
 
     if (extension->priv->key_file)
     {
-        GError* error = NULL;
-        /* FIXME: Handle readonly folder/ file */
-        gchar* config_file = g_build_filename (extension->priv->config_dir,
-                                               "config", NULL);
-        katze_mkdir_with_parents (extension->priv->config_dir, 0700);
         g_key_file_set_string_list (extension->priv->key_file,
                                     "settings", name, (const gchar**)value, length);
-        sokoke_key_file_save_to_file (extension->priv->key_file, config_file, &error);
-        if (error)
-        {
-            printf (_("The configuration of the extension '%s' couldn't be saved: %s\n"),
-                    extension->priv->name, error->message);
-            g_error_free (error);
-        }
+        midori_extension_save_settings (extension);
     }
 }
