@@ -400,42 +400,6 @@ midori_search_action_manage_activate_cb (GtkWidget*          menuitem,
         gtk_widget_show (dialog);
 }
 
-GdkPixbuf*
-midori_search_action_get_icon (KatzeItem*    item,
-                               GtkWidget*    widget,
-                               const gchar** icon_name,
-                               gboolean      in_entry)
-{
-    const gchar* icon = katze_item_get_uri (item);
-    GdkScreen* screen;
-    GtkIconTheme* icon_theme;
-
-    if (midori_uri_is_resource (icon))
-        return katze_load_cached_icon (icon, widget);
-
-    if (icon_name == NULL)
-        return NULL;
-
-    screen = gtk_widget_get_screen (widget);
-    icon_theme = gtk_icon_theme_get_for_screen (screen);
-    if ((icon = katze_item_get_icon (item)) && *icon)
-    {
-        if (gtk_icon_theme_has_icon (icon_theme, icon))
-        {
-            *icon_name = icon;
-            return NULL;
-        }
-    }
-
-    if (in_entry && gtk_icon_theme_has_icon (icon_theme, "edit-find-option-symbolic"))
-        *icon_name = "edit-find-option-symbolic";
-    else if (gtk_icon_theme_has_icon (icon_theme, "edit-find-option"))
-        *icon_name = "edit-find-option";
-    else
-        *icon_name = STOCK_EDIT_FIND;
-    return NULL;
-}
-
 static void
 midori_search_action_icon_released_cb (GtkWidget*           entry,
                                        GtkIconEntryPosition icon_pos,
@@ -463,14 +427,13 @@ midori_search_action_icon_released_cb (GtkWidget*           entry,
             menuitem = gtk_image_menu_item_new_with_label (
                 katze_item_get_name (item));
             image = gtk_image_new ();
-            icon = midori_search_action_get_icon (item, entry, &icon_name, FALSE);
-            if (icon)
+            if ((icon = katze_item_get_pixbuf (item, entry)))
             {
                 gtk_image_set_from_pixbuf (GTK_IMAGE (image), icon);
                 g_object_unref (icon);
             }
             else
-                gtk_image_set_from_icon_name (GTK_IMAGE (image), icon_name,
+                gtk_image_set_from_icon_name (GTK_IMAGE (image), STOCK_EDIT_FIND,
                                               GTK_ICON_SIZE_MENU);
             gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
             gtk_image_menu_item_set_always_show_image (
@@ -523,19 +486,25 @@ midori_search_action_set_entry_icon (MidoriSearchAction* search_action,
 
     if (search_action->current_item)
     {
-        const gchar* icon_name;
-
-        icon = midori_search_action_get_icon (search_action->current_item,
-                                              entry, &icon_name, TRUE);
-        if (icon)
+        if ((icon = katze_item_get_pixbuf (search_action->current_item, entry)))
         {
-            gtk_icon_entry_set_icon_from_pixbuf (GTK_ICON_ENTRY (entry),
-                                                 GTK_ICON_ENTRY_PRIMARY, icon);
+            gtk_entry_set_icon_from_pixbuf (GTK_ENTRY (entry), GTK_ENTRY_ICON_PRIMARY, icon);
             g_object_unref (icon);
         }
         else
+        {
+            GdkScreen* screen = gtk_widget_get_screen (entry);
+            GtkIconTheme* icon_theme = gtk_icon_theme_get_for_screen (screen);
+            gchar* icon_name;
+            if (gtk_icon_theme_has_icon (icon_theme, "edit-find-option-symbolic"))
+                icon_name = "edit-find-option-symbolic";
+            else if (gtk_icon_theme_has_icon (icon_theme, "edit-find-option"))
+                icon_name = "edit-find-option";
+            else
+                icon_name = STOCK_EDIT_FIND;
             gtk_icon_entry_set_icon_from_icon_name (GTK_ICON_ENTRY (entry),
                 GTK_ICON_ENTRY_PRIMARY, icon_name);
+        }
         gtk_entry_set_placeholder_text (GTK_ENTRY (entry),
             katze_item_get_name (search_action->current_item));
     }
@@ -813,17 +782,16 @@ midori_search_action_dialog_render_icon_cb (GtkTreeViewColumn* column,
 {
     KatzeItem* item;
     GdkPixbuf* icon;
-    const gchar* icon_name;
 
     gtk_tree_model_get (model, iter, 0, &item, -1);
 
-    if ((icon = midori_search_action_get_icon (item, treeview, &icon_name, FALSE)))
+    if ((icon = katze_item_get_pixbuf (item, treeview)))
     {
         g_object_set (renderer, "pixbuf", icon, "yalign", 0.25, NULL);
         g_object_unref (icon);
     }
     else
-        g_object_set (renderer, "icon-name", icon_name, "yalign", 0.25, NULL);
+        g_object_set (renderer, "icon-name", STOCK_EDIT_FIND, "yalign", 0.25, NULL);
     g_object_unref (item);
 }
 
@@ -951,7 +919,7 @@ midori_search_action_get_engine_for_form (WebKitWebView*     web_view,
     gchar** parts;
 
     #if WEBKIT_CHECK_VERSION (1, 9, 5)
-    doc = webkit_web_frame_get_dom_document (web_view);
+    doc = webkit_web_frame_get_dom_document (webkit_web_view_get_focused_frame (web_view));
     #else
     if (webkit_web_view_get_focused_frame (web_view) != webkit_web_view_get_main_frame (web_view))
         return NULL;
@@ -1013,7 +981,8 @@ midori_search_action_get_engine_for_form (WebKitWebView*     web_view,
         }
     }
 
-    title = webkit_web_view_get_title (web_view);
+    if (!(title = webkit_web_view_get_title (web_view)))
+        title = webkit_web_view_get_uri (web_view);
 
     item = katze_item_new ();
     item->uri = g_string_free (uri_str, FALSE);
@@ -1395,14 +1364,6 @@ midori_search_action_treeview_destroy_cb (GtkWidget*          treeview,
     }
 }
 
-static void
-midori_search_action_dialog_respnse_cb (GtkWidget* dialog,
-                                        gint       response,
-                                        gpointer   data)
-{
-    gtk_widget_destroy (dialog);
-}
-
 /**
  * midori_search_action_get_dialog:
  * @search_action: a #MidoriSearchAction
@@ -1451,9 +1412,7 @@ midori_search_action_get_dialog (MidoriSearchAction* search_action)
         toplevel ? GTK_WINDOW (toplevel) : NULL,
         GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
         #if !HAVE_OSX
-        #if !HAVE_HILDON
         GTK_STOCK_HELP, GTK_RESPONSE_HELP,
-        #endif
         GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
         #endif
         NULL);
@@ -1468,8 +1427,8 @@ midori_search_action_get_dialog (MidoriSearchAction* search_action)
                                        GTK_RESPONSE_HELP, FALSE);
     sokoke_widget_get_text_size (dialog, "M", &width, &height);
     gtk_window_set_default_size (GTK_WINDOW (dialog), width * 52, -1);
-    g_signal_connect (dialog, "response",
-                      G_CALLBACK (midori_search_action_dialog_respnse_cb), NULL);
+    g_signal_connect_swapped (dialog, "response",
+        G_CALLBACK (gtk_widget_destroy), dialog);
     /* TODO: Do we want tooltips for explainations or can we omit that?
              We need mnemonics */
     if ((xfce_heading = sokoke_xfce_header_new (
@@ -1592,3 +1551,159 @@ midori_search_action_get_dialog (MidoriSearchAction* search_action)
     search_action->dialog = dialog;
     return dialog;
 }
+
+KatzeArray*
+midori_search_engines_new_from_file (const gchar* filename,
+                                     GError**     error)
+{
+    KatzeArray* search_engines;
+    GKeyFile* key_file;
+    gchar** engines;
+    guint i, j, n_properties;
+    KatzeItem* item;
+    GParamSpec** pspecs;
+    const gchar* property;
+    gchar* value;
+
+    search_engines = katze_array_new (KATZE_TYPE_ITEM);
+    key_file = g_key_file_new ();
+    g_key_file_load_from_file (key_file, filename,
+                               G_KEY_FILE_KEEP_COMMENTS, error);
+    engines = g_key_file_get_groups (key_file, NULL);
+    pspecs = g_object_class_list_properties (G_OBJECT_GET_CLASS (search_engines),
+                                             &n_properties);
+    for (i = 0; engines[i] != NULL; i++)
+    {
+        item = katze_item_new ();
+        for (j = 0; j < n_properties; j++)
+        {
+            if (!G_IS_PARAM_SPEC_STRING (pspecs[j]))
+                continue;
+            property = g_param_spec_get_name (pspecs[j]);
+            value = g_key_file_get_string (key_file, engines[i], property, NULL);
+            g_object_set (item, property, value, NULL);
+            g_free (value);
+        }
+        katze_array_add_item (search_engines, item);
+    }
+    g_free (pspecs);
+    g_strfreev (engines);
+    g_key_file_free (key_file);
+    return search_engines;
+}
+
+KatzeArray*
+midori_search_engines_new_from_folder (GString* error_messages)
+{
+    gchar* config_file = midori_paths_get_config_filename_for_reading ("search");
+    GError* error = NULL;
+    KatzeArray* search_engines = midori_search_engines_new_from_file (config_file, &error);
+    /* We ignore for instance empty files */
+    if (error && (error->code == G_KEY_FILE_ERROR_PARSE
+        || error->code == G_FILE_ERROR_NOENT))
+    {
+        g_error_free (error);
+        error = NULL;
+    }
+    if (!error && katze_array_is_empty (search_engines))
+    {
+        katze_assign (config_file,
+            midori_paths_get_preset_filename (NULL, "search"));
+        katze_object_assign (search_engines,
+            midori_search_engines_new_from_file (config_file, NULL));
+    }
+    else if (error)
+    {
+        if (error->code != G_FILE_ERROR_NOENT && error_messages)
+            g_string_append_printf (error_messages,
+                _("The search engines couldn't be loaded. %s\n"),
+                error->message);
+        g_error_free (error);
+    }
+    g_free (config_file);
+    return search_engines;
+}
+
+gboolean
+midori_search_engines_save_to_file (KatzeArray*  search_engines,
+                                    const gchar* filename,
+                                    GError**     error)
+{
+    GKeyFile* key_file;
+    guint j, n_properties;
+    KatzeItem* item;
+    const gchar* name;
+    GParamSpec** pspecs;
+    const gchar* property;
+    gchar* value;
+    gboolean saved;
+
+    key_file = g_key_file_new ();
+    pspecs = g_object_class_list_properties (G_OBJECT_GET_CLASS (search_engines),
+                                             &n_properties);
+    KATZE_ARRAY_FOREACH_ITEM (item, search_engines)
+    {
+        name = katze_item_get_name (item);
+        for (j = 0; j < n_properties; j++)
+        {
+            if (!G_IS_PARAM_SPEC_STRING (pspecs[j]))
+                continue;
+            property = g_param_spec_get_name (pspecs[j]);
+            g_object_get (item, property, &value, NULL);
+            if (value)
+                g_key_file_set_string (key_file, name, property, value);
+            g_free (value);
+        }
+    }
+    g_free (pspecs);
+    saved = sokoke_key_file_save_to_file (key_file, filename, error);
+    g_key_file_free (key_file);
+
+    return saved;
+}
+
+static void
+midori_search_engines_modify_cb (KatzeArray* array,
+                                 gpointer    item,
+                                 KatzeArray* search_engines)
+{
+    gchar* config_file = g_object_get_data (G_OBJECT (search_engines), "search-engines-filename");
+    GError* error = NULL;
+    if (!midori_search_engines_save_to_file (search_engines, config_file, &error))
+    {
+        g_warning (_("The search engines couldn't be saved. %s"),
+                   error->message);
+        g_error_free (error);
+    }
+}
+
+static void
+midori_search_engines_move_item_cb (KatzeArray* array,
+                                    gpointer    item,
+                                    gint        position,
+                                    KatzeArray* search_engines)
+{
+    midori_search_engines_modify_cb (array, item, search_engines);
+}
+
+void
+midori_search_engines_set_filename (KatzeArray*  search_engines,
+                                    const gchar* filename)
+{
+    g_object_set_data_full (G_OBJECT (search_engines), "search-engines-filename",
+                            g_strdup (filename), (GDestroyNotify)g_free);
+    g_signal_connect_after (search_engines, "add-item",
+        G_CALLBACK (midori_search_engines_modify_cb), search_engines);
+    g_signal_connect_after (search_engines, "remove-item",
+        G_CALLBACK (midori_search_engines_modify_cb), search_engines);
+    if (!katze_array_is_empty (search_engines))
+    {
+        KatzeItem* item;
+        KATZE_ARRAY_FOREACH_ITEM (item, search_engines)
+            g_signal_connect_after (item, "notify",
+                G_CALLBACK (midori_search_engines_modify_cb), search_engines);
+        g_signal_connect_after (search_engines, "move-item",
+            G_CALLBACK (midori_search_engines_move_item_cb), search_engines);
+    }
+}
+
