@@ -285,6 +285,8 @@ magic_uri_format (void)
      { "http://wiki.c3sl.ufpr.br/multiseat/index.php/Xephyr_Solution", NULL },
      { "http://şøñđëřżēıċħęŋđőmæîņĭśŧşũþėŗ.de/char.jpg", NULL },
      { "http://www.ⓖⓝⓞⓜⓔ.org/", "http://www.gnome.org/" },
+     { "http://translate.google.com/#en/de/cat%0Adog%0Ahorse",
+       "http://translate.google.com/#en/de/cat%0Adog%0Ahorse" },
      };
     guint i;
 
@@ -297,6 +299,33 @@ magic_uri_format (void)
     }
 }
 
+static void
+magic_uri_prefix (void)
+{
+
+    typedef struct
+    {
+        const gchar* before;
+        const gchar* after;
+    } URIItem;
+
+    static const URIItem items[] = {
+        { "http://www.example.com", "example.com" },
+        { "http://example.com", "example.com" },
+        { "https://example.com", "example.com" },
+        { "file:///path/to/my/heart", "/path/to/my/heart" },
+     };
+    guint i;
+
+    for (i = 0; i < G_N_ELEMENTS (items); i++)
+    {
+        gchar* result = midori_uri_strip_prefix_for_display (items[i].before);
+        const gchar* after = items[i].after ? items[i].after : items[i].before;
+        katze_assert_str_equal (items[i].before, result, after);
+        g_free (result);
+    }
+}
+
 static void
 magic_uri_prefetch (void)
 {
@@ -362,7 +391,7 @@ main (int    argc,
       char** argv)
 {
     g_test_init (&argc, &argv, NULL);
-    midori_app_setup (&argc, &argv, NULL, NULL);
+    midori_app_setup (&argc, &argv, NULL);
 
     g_test_add_func ("/magic-uri/uri", magic_uri_uri);
     g_test_add_func ("/magic-uri/idn", magic_uri_idn);
@@ -372,6 +401,7 @@ main (int    argc,
     g_test_add_func ("/magic-uri/fingerprint", magic_uri_fingerprint);
     g_test_add_func ("/magic-uri/ip", magic_uri_ip);
     g_test_add_func ("/magic-uri/format", magic_uri_format);
+    g_test_add_func ("/magic-uri/prefix", magic_uri_prefix);
     g_test_add_func ("/magic-uri/prefetch", magic_uri_prefetch);
     g_test_add_func ("/magic-uri/commands", magic_uri_commands);
 
